@@ -1,10 +1,13 @@
 /*
-
   This is an encapsulation of the  Netscape plugin API.
 
   Copyright (c) 2000 Matthias Hoelzer-Kluepfel <hoelzer@kde.org>
                      Stefan Schimanski <1Stein@gmx.de>
-  Copyright (c) 2003-2005 George Staikos <staikos@kde.org>
+                2003-2005 George Staikos <staikos@kde.org>
+                2007, 2008 Maksim Orlovich     <maksim@kde.org>
+                2006, 2007, 2008 Apple Inc.
+                2008 Collabora, Ltd.
+                2008 Sebastian Sauer <mail@dipe.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -19,49 +22,53 @@
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-
 */
 
-
 #ifndef __NS_PLUGIN_H__
 #define __NS_PLUGIN_H__
 
+#include <QObject>
 
-#include <dcopobject.h>
-#include "NSPluginClassIface.h"
-#include "NSPluginCallbackIface_stub.h"
+#include <QMap>
+#include <QPointer>
+#include <QQueue>
+#include <QList>
 
+#include <KDebug>
 
-#include <qobject.h>
-#include <qstring.h>
-#include <qstringlist.h>
-#include <qptrqueue.h>
-#include <qdict.h>
-#include <qmap.h>
-#include <qintdict.h>
-#include <qguardedptr.h>
-
-#include <kparts/browserextension.h>  // for URLArgs
+#include <kparts/browserextension.h>  // for OpenUrlArguments
 #include <kio/job.h>
+#include <QDBusObjectPath>
 
+#include <QX11EmbedWidget>
 
 #define XP_UNIX
 #define MOZ_X11
 #include "sdk/npupp.h"
 
+#include "comm/dbustypes.h"
+
 typedef char* NP_GetMIMEDescriptionUPP(void);
 typedef NPError NP_InitializeUPP(NPNetscapeFuncs*, NPPluginFuncs*);
 typedef NPError NP_ShutdownUPP(void);
 
-
 #include <X11/Intrinsic.h>
-
+#include <fixx11h.h>
 
 void quitXt();
 
+class OrgKdeNspluginsCallBackInterface;
 class KLibrary;
 class QTimer;
 
+// We need the following inside the scripting code
+namespace kdeNsPluginViewer {
+  void* g_NPN_MemAlloc(uint32 size);
+  void  g_NPN_MemFree(void* ptr);
+  class ScriptExportEngine;
+}
+
+using namespace kdeNsPluginViewer;
 
 class NSPluginStreamBase : public QObject
 {
@@ -71,11 +78,11 @@ public:
   NSPluginStreamBase( class NSPluginInstance *instance );
   ~NSPluginStreamBase();
 
-  KURL url() { return _url; }
+  KUrl url() { return _url; }
   int pos() { return _pos; }
   void stop();
 
-signals:
+Q_SIGNALS:
   void finished( NSPluginStreamBase *strm );
 
 protected:
@@ -86,24 +93,24 @@ protected:
   bool create( const QString& url, const QString& mimeType, void *notify, bool forceNotify = false );
   int tries() { return _tries; }
   void inform( );
-  void updateURL( const KURL& newURL );
+  void updateURL( const KUrl& newUrl );
 
   class NSPluginInstance *_instance;
   uint16 _streamType;
   NPStream *_stream;
   void *_notifyData;
-  KURL _url;
+  KUrl _url;
   QString _fileURL;
   QString _mimeType;
   QByteArray _data;
-  class KTempFile *_tempFile;
+  class KTemporaryFile *_tempFile;
 
 private:
   int process( const QByteArray &data, int start );
 
   unsigned int _pos;
   QByteArray _queue;
-  unsigned int _queuePos;
+  int _queuePos;
   int _tries;
   bool _onlyAsFile;
   bool _error;
@@ -120,19 +127,21 @@ public:
   NSPluginStream( class NSPluginInstance *instance );
   ~NSPluginStream();
 
-  bool get(const QString& url, const QString& mimeType, void *notifyData, bool reload = false);
-  bool post(const QString& url, const QByteArray& data, const QString& mimeType, void *notifyData, const KParts::URLArgs& args);
+    bool get(const QString& url, const QString& mimeType, void *notifyData, bool reload = false);
+    bool post(const QString& url, const QByteArray& data, const QString& mimeType, void *notifyData,
+              const KParts::OpenUrlArguments& args,
+              const KParts::BrowserArguments& browserArgs);
 
-protected slots:
+protected Q_SLOTS:
   void data(KIO::Job *job, const QByteArray &data);
-  void totalSize(KIO::Job *job, KIO::filesize_t size);
+  void totalSize(KJob *job, qulonglong size);
   void mimetype(KIO::Job * job, const QString &mimeType);
-  void result(KIO::Job *job);
-  void redirection(KIO::Job *job, const KURL& url);
+  void result(KJob *job);
+  void redirection(KIO::Job *job, const KUrl& url);
   void resume();
 
 protected:
-  QGuardedPtr<KIO::TransferJob> _job;
+  QPointer<KIO::TransferJob> _job;
   QTimer *_resumeTimer;
 };
 
@@ -147,7 +156,7 @@ public:
 
   bool get( const QString& url, const QString& mimeType, const QByteArray &buf, void *notifyData, bool singleShot=false );
 
-protected slots:
+protected Q_SLOTS:
   void timer();
 
 protected:
@@ -155,30 +164,45 @@ protected:
   bool _singleShot;
 };
 
+class PluginHost;
 
-class NSPluginInstance : public QObject, public virtual NSPluginInstanceIface
+class NSPluginInstance : public QObject
 {
   Q_OBJECT
 
 public:
 
   // constructor, destructor
-  NSPluginInstance( NPP privateData, NPPluginFuncs *pluginFuncs, KLibrary *handle,
-		    int width, int height, QString src, QString mime,
-                    QString appId, QString callbackId, bool embed, WId xembed,
-		    QObject *parent, const char* name=0 );
+  NSPluginInstance( NPPluginFuncs *pluginFuncs, KLibrary *handle,
+                    const QString &src, const QString &mime,
+                    const QStringList &argn, const QStringList &argv,
+                    const QString &appId, const QString &callbackId, bool embed,
+		    QObject *parent );
   ~NSPluginInstance();
 
-  // DCOP functions
+  bool wasInitializedOK() { return _initializedOK; }
+
+  // DBus-exported functions
   void shutdown();
-  int winId() { return _form != 0 ? XtWindow(_form) : 0; }
-  int setWindow(Q_INT8 remove=0);
-  void resizePlugin(Q_INT32 w, Q_INT32 h);
-  void javascriptResult(Q_INT32 id, QString result);
-  void displayPlugin();
+  void setupWindow(int winId, int w, int h);
+  void resizePlugin(int clientWinId, int w, int h);
+  void javascriptResult(int id, const QString &result);
   void gotFocusIn();
   void gotFocusOut();
 
+  // d-bus export scripting interface
+  NSLiveConnectResult lcCall(qulonglong obj, const QString& f, const QStringList& args);
+  NSLiveConnectResult lcGet (qulonglong obj, const QString& f);
+  bool lcPut(qulonglong obj, const QString& f, const QString& v);
+  void lcUnregister(qulonglong obj);
+
+  ScriptExportEngine* scripting() { return _scripting; }
+  
+
+  // last via NSPluginClass::newInstance() produced NSPluginInstance instance.
+  static NSPluginInstance* lastPluginInstance();
+  static void setLastPluginInstance(NSPluginInstance*);
+
   // value handling
   NPError NPGetValue(NPPVariable variable, void *value);
   NPError NPSetValue(NPNVariable variable, void *value);
@@ -194,7 +218,7 @@ public:
   int32 NPWriteReady(NPStream *stream);
 
   // URL functions
-  void NPURLNotify(QString url, NPReason reason, void *notifyData);
+  void NPURLNotify(const QString &url, NPReason reason, void *notifyData);
 
   // Event handling
   uint16 HandleEvent(void *event);
@@ -204,39 +228,43 @@ public:
   void requestURL( const QString &url, const QString &mime,
 		   const QString &target, void *notify, bool forceNotify = false, bool reload = false );
   void postURL( const QString &url, const QByteArray& data, const QString &mime,
-             const QString &target, void *notify, const KParts::URLArgs& args, bool forceNotify = false );
+                const QString &target, void *notify, const KParts::OpenUrlArguments& args,
+                const KParts::BrowserArguments& browserArgs, bool forceNotify = false );
 
   QString normalizedURL(const QString& url) const;
+  
+  bool hasPendingJSRequests() const;
 
-public slots:
+  QString pageURL() const { return _pageURL; }
+
+public Q_SLOTS:
   void streamFinished( NSPluginStreamBase *strm );
 
-private slots:
   void timer();
 
 private:
   friend class NSPluginStreamBase;
-
-  static void forwarder(Widget, XtPointer, XEvent *, Boolean*);
+  friend class kdeNsPluginViewer::ScriptExportEngine;
 
   void destroy();
 
   bool _destroyed;
-  bool _visible;
-  void addTempFile(KTempFile *tmpFile);
-  QPtrList<KTempFile> _tempFiles;
-  NSPluginCallbackIface_stub *_callback;
-  QPtrList<NSPluginStreamBase> _streams;
+  bool _embedded;
+  void addTempFile(KTemporaryFile *tmpFile);
+  QList<KTemporaryFile *> _tempFiles;
+  OrgKdeNspluginsCallBackInterface *_callback;
+  QList<NSPluginStreamBase *> _streams;
   KLibrary *_handle;
   QTimer *_timer;
 
   NPP      _npp;
   NPPluginFuncs _pluginFuncs;
 
-  Widget _area, _form, _toplevel;
-  WId _xembed_window;
+  PluginHost*      _pluginHost; // Manages embedding of the plugin into us
+  int _width, _height;          // last size we used;
+
   QString _baseURL;
-  int _width, _height;
+  QString _pageURL;
 
   struct Request
   {
@@ -249,10 +277,14 @@ private:
       // A POST request
       Request( const QString &_url, const QByteArray& _data,
                const QString &_mime, const QString &_target, void *_notify,
-               const KParts::URLArgs& _args, bool _forceNotify = false)
-	  { url=_url; mime=_mime; target=_target;
-            notify=_notify; post=true; data=_data; args=_args;
-            forceNotify = _forceNotify; }
+               const KParts::OpenUrlArguments& _args,
+               const KParts::BrowserArguments& _browserArgs,
+               bool _forceNotify = false)
+      {
+          url=_url; mime=_mime; target=_target;
+          notify=_notify; post=true; data=_data; args=_args; browserArgs=_browserArgs;
+          forceNotify = _forceNotify;
+      }
 
       QString url;
       QString mime;
@@ -262,36 +294,41 @@ private:
       bool forceNotify;
       bool reload;
       void *notify;
-      KParts::URLArgs args;
+      KParts::OpenUrlArguments args;
+      KParts::BrowserArguments browserArgs;
   };
 
-  NPWindow _win;
-  NPSetWindowCallbackStruct _win_info;
-  QPtrQueue<Request> _waitingRequests;
+  QQueue<Request *> _waitingRequests;
   QMap<int, Request*> _jsrequests;
+  int _numJSRequests; // entered in earlier than _jsrequests.
+
+  ScriptExportEngine* _scripting; //0 if plugin doesn't support it.
+
+  bool _initializedOK; // ok/fail from ctor
+  
+  static NSPluginInstance* s_lastPluginInstance;
 };
 
 
-class NSPluginClass : public QObject, virtual public NSPluginClassIface
+class NSPluginClass : public QObject
 {
   Q_OBJECT
 public:
 
-  NSPluginClass( const QString &library, QObject *parent, const char *name=0 );
+  NSPluginClass( const QString &library, QObject *parent );
   ~NSPluginClass();
 
   QString getMIMEDescription();
-  DCOPRef newInstance(QString url, QString mimeType, Q_INT8 embed,
-                      QStringList argn, QStringList argv,
-                      QString appId, QString callbackId, Q_INT8 reload, Q_INT8 post,
-                      QByteArray postData, Q_UINT32 xembed );
+  QDBusObjectPath newInstance(const QString &url, const QString &mimeType, bool embed,
+                      const QStringList &argn, const QStringList &argv,
+                      const QString &appId, const QString &callbackId, bool reload);
   void destroyInstance( NSPluginInstance* inst );
   bool error() { return _error; }
 
-  void setApp(const QCString& app) { _app = app; }
-  const QCString& app() const { return _app; }
+  void setApp(const QByteArray& app) { _app = app; }
+  const QByteArray& app() const { return _app; }
 
-protected slots:
+protected Q_SLOTS:
   void timer();
 
 private:
@@ -308,31 +345,34 @@ private:
   NP_InitializeUPP *_NP_Initialize;
   NP_ShutdownUPP *_NP_Shutdown;
 
+  QList<NSPluginInstance *> _instances;
+  QList<NSPluginInstance *> _trash;
+
+  QByteArray _app;
   NPPluginFuncs _pluginFuncs;
   NPNetscapeFuncs _nsFuncs;
 
-  QPtrList<NSPluginInstance> _instances;
-  QPtrList<NSPluginInstance> _trash;
-
-  QCString _app;
+  // If plugins use gtk, we call the gtk_init function for them ---
+  // but only do it once.
+  static bool s_initedGTK;
 };
 
 
-class NSPluginViewer : public QObject, virtual public NSPluginViewerIface
+class NSPluginViewer : public QObject
 {
     Q_OBJECT
 public:
-   NSPluginViewer( QCString dcopId, QObject *parent, const char *name=0 );
+   NSPluginViewer( QObject *parent );
    virtual ~NSPluginViewer();
 
    void shutdown();
-   DCOPRef newClass( QString plugin );
+   QDBusObjectPath newClass( const QString& plugin, const QString& senderId );
 
-private slots:
-   void appUnregistered(const QCString& id);
+private Q_SLOTS:
+   void appChanged( const QString& id,  const QString& oldOwner, const QString& newOwner);
 
 private:
-   QDict<NSPluginClass> _classes;
+   QMap<QString, NSPluginClass *> _classes;
 };
 
 
